cimport numpy as np
import numpy as np

cdef extern from "numpy/ndarraytypes.h":
    void PyArray_CLEARFLAGS(np.ndarray arr, int flags)
    void PyArray_ENABLEFLAGS(np.ndarray arr, int flags)

cpdef remove_ownership(np.ndarray arr):    
    PyArray_CLEARFLAGS(arr, np.NPY_OWNDATA)

cpdef give_ownership(np.ndarray arr):    
    PyArray_ENABLEFLAGS(arr, np.NPY_OWNDATA)




from __future__ import print_function
import gzip
import os
import pickle
import sys
import subprocess

PYFILE_VERSION = 1

class VersionError(Exception):
    pass


def loadData(filename):
    try:
        with gzip.open(filename) as f:
            x = f.read()
    except OSError as e:
        e.args = [filename + " is not a valid .mod file"]
        raise
    return parseInput(x, filename)

def hashFile(filename):
    p = subprocess.check_output(["md5sum", filename])
    return p.decode().split()[0]

def parseInput(x, filename):
    res = {}
    
    x = x.decode()
    header = x.split('\n')[0]
    x = x.replace('\n', ' ')
    
    if 'GFORTRAN' not in header:
            raise ValueError('Not a gfortran mod file')
    
    res['version'] = int(header.split("'")[1])
    res['orig_file'] = header.split()[-1]
    res['filename'] = filename
    res['checksum'] = hashFile(filename)
    
    if res['version'] == 14:
            from . import parseMod14 as p
    else:
            raise VersionError("Only supports mod version 14")
    
    pm = p.parseMod(x,filename,res,PYFILE_VERSION)
    
    return pm
	
def run(filename,output=None,save=True,unpack=True):
    x = loadData(filename)
    x.processData()
    if save:
        if output is None:
            output=fpyname(filename)
        
        x.save(output)
        
    if unpack:
        return x.getUnpackedData()
    else:
        return x

def fpyname(filename):
    return filename.split('.')[0] + '.fpy'

#################################

if __name__ == "__main__":
    if len(sys.argv[1:]) > 0:
        files = sys.argv[1:]
        
    for filename in files:
        run(filename,save=True,unpack=False)



from __future__ import print_function
try:
    import __builtin__ 
except ImportError:
    import builtins as __builtin__
import gzip
import ctypes
import os
import pickle
import sys
import re
import subprocess
import numpy as np
import multiprocessing as mp

from .utils import *
from .utils_cpython import *

PARALLEL = False

class parseModBase(object):
    def __init__(self,data,filename,res,PYFILE_VERSION):
        self.data = data
        self.filename = filename
        self.mod_data = res
        self.PYFILE_VERSION = PYFILE_VERSION
        
        self.funcs=[]
        self.mod_vars=[]
        self.param=[]
        self.dt_defs=[]
        self.dt_defines=[]
        self._unpacked=False

    def processData(self):
        self.data = split_brackets(self.data)
        
        #Store the split data once:
        _t=[]
        for i in self.data:
            _t.append(split_brackets(i.strip(),remove_b=True))
        self.data = _t
        
    
        self.parseAllIntrinsic()
        self.parseAllOperator()
        self.parseAllDTDefines()
        self.parseAllCommon()
        self.parseAllEqv()
        self.parseAllNameSymbols()
        self.parseAllSymbols()
        
        self.matchFuncArgs()
        self.data=None
        
        self.unpackData()

    def save(self,output):
        if not self._unpacked:
            self.unpackData()
        
        self.pickler(output, self.PYFILE_VERSION, 
                    self.mod_data, self.mod_vars, self.param, 
                    self.funcs, self.dt_defs)

 
    def pickler(self,filename, *args):   
        with open(filename, 'wb') as f:
            for i in args:
                pickle.dump(i, f, protocol=2)
            
    def unpackData(self):
        self._unpacked = True
        
        for i in self.all_symbols:
            if 'proc' in i:
                self.funcs.append(i)
            elif 'var' in i:
                self.mod_vars.append(i)
            elif 'param' in i:
                self.param.append(i)
            elif 'dt_def' in i:
                self.dt_defs.append(i)
                
    def getUnpackedData(self):
        if not self._unpacked:
            self.unpackData()
            
        return  self.mod_data, self.mod_vars, self.param, self.funcs, self.dt_defs
    
    def mangleName(self,item):
        return '__' + item['module'] + '_MOD_' + item['name'].lower()
        
    def parseAllIntrinsic(self):
        x = self.data[0]
        #Remove the module header
        #x = x[x.index("("):]
        
    def parseAllOperator(self):
        x = self.data[1]
        
    def parseAllDTDefines(self):
        # This isnt just dt names, its also the name and number(s) for 
        # generic interfaces
        x = self.data[2]
        #x = split_brackets(x.strip(),remove_b=True) 
        
        for i in x:
            i=i.replace("(","").replace(")",'').strip()
            r={}
            z = i.split()
            r['name'] = z[0].lower().replace("'","")
            r['module'] = z[1]
            r['num'] = int(z[2])
            if(len(z))> 3:
                r['num_alts']=z[2:]
            self.dt_defines.append(r)
            
            
        # Find the dt's used by the module but not defined by the module,
        # but its defintion is still copied in here
        for symbol in self.data[6]:
            try:
                splitPoint = symbol.index("(")
            except ValueError:
                continue
            info = symbol[splitPoint:].strip()
            info = split_brackets(info)
            type_line = info[0]
            
            if 'DERIVED ' in type_line:
                num,name,module  =  symbol.split()[0:3]
                if len(self.dt_defines)>0:
                    found=False
                    for i in self.dt_defines:
                        if int(i['num']) == int(num):
                            found=True
                else:
                    found = False
                
                if not found:
                    self.dt_defines.append({'name':name,
                                        'module':module,
                                        'num':num})
    
    def parseAllCommon(self):
        x = self.data[3]
        
    def parseAllEqv(self):
        x = self.data[4]

    def parseAllNameSymbols(self):
        x = self.data[7]
        x = x[0].split()
        self.symbol_names = []
        for i in range(len(x)//3):
            self.symbol_names.append({'name':x[i*3],'num':x[i*3+2],'ambiguous_flag':bool(x[i*3+1])})
        
    
    def parseAllSymbols(self):
        split_data = self.data[6]
        
        if PARALLEL:
            with mp.Pool() as pool:
                all_symbols = pool.map(self.parseSymbol,split_data)
        else:
            all_symbols = [self.parseSymbol(i) for i in split_data]
            
        self.all_symbols = all_symbols
        
    def parseSymbol(self,symbol):
        symbol = symbol.strip()
        #things to skip

        if ('__' in symbol or '(intrinsic)' in symbol or 
            'INTRINSIC' in symbol or len(symbol)==0 or
            ' RESULT' in symbol or 'BODY' in symbol):
            return {}

        r = {}
        r['num'], r['name'],r['module'], _, r['parent_id']  =  symbol.split()[0:5]
        
        for i in ['num','name','module','parent_id']:
            r[i]=r[i].lower().replace("'","")
        
        if len(r['name'])==0:
            return {}
        
        ## things to left of '(' are basic info right of '(' contains detailed info
        ## about the symbol  
        splitPoint = symbol.index("(")
        
        info = symbol[splitPoint:].strip()
        
        
        info = split_brackets(info)
        r['info'] = info
        type_line = info[0]
        # need spaces on end of names as sometimes we have name-something etc
        if 'VARIABLE ' in type_line or 'DUMMY ' in type_line:
            if len(r['module'])>0:
                r['var'] = self.parseVar(info)
            else:
                r['var'] = self.parseFuncArg(info)
                r['func_arg']=True
        elif 'PROCEDURE ' in type_line:
            if 'GENERIC' in type_line:
                pass #Uneeded
                #r['dt_type'] = self.parseDTType(info)
            else:
                r['proc'] = self.parseProc(info)
                r['arg']=[]
        elif 'MODULE ' in type_line:
            r['module_info'] = self.parseModule(info)
        elif 'PARAMETER ' in type_line:
            r['param'] = self.parseParam(info)
        elif 'DERIVED ' in type_line:
            #This definition of a dt 
            r['dt_def'] = self.parseDT(info)
        elif 'NAMELIST ' in type_line:
            print("Skipping namelist "+r['name'])
        else:
            raise ValueError("Unknown object "+symbol)
            
        r['mangled_name']=self.mangleName(r)
            
        r.pop('info')

        return r 
        
       
    def parseVar(self,info):
        res={}

        type_info = info[2]
        symbol_info = info[0]
        p, c, s=self.getVarType(type_info)
        
        res['pytype'] = p
        res['ctype'] = c
        res['bytes'] = s
        
        if res['pytype'] == 'str':
            res['len'] = self.getStrLen(info)
        
        if 'DIMENSION' in symbol_info:
            #Array
            res['array'] = self.processArray(info)
        if 'DERIVED' in type_info:
            #Both dts but sometimes the definition moves
            res['dt'] = self.processDTVar(info)
            
        if 'POINTER' in symbol_info:
            res['pointer']=True
        if 'TARGET' in symbol_info:
            res['target']=True
        if 'OPTIONAL' in symbol_info:
            res['optional']=True
        if 'DUMMY' in symbol_info:
            res['intent'] = self.parseIntent(symbol_info)

        return res
        
    def parseProc(self,info):
        res={}
        type_info = info[2]
        
        if 'FUNCTION ' in info[0]:        
            p, c, s=self.getVarType(type_info)
        else:
            p = None
            c = None
            s = None
            
        res['pytype'] = p
        res['ctype'] = c
        res['bytes'] = s
        
        x=info[3][info[3].index("("):]
        res['arg_nums']=x.replace("(","").replace(")","").split()
        
        if 'SUBROUTINE' in info[0]:
            res['sub']=True
        else:
            res['sub']=False
        
        #Return value
        res['ret'] = self.parseVar(info)
        
        func_args = info[3].split()
        res['func_arg_id'] = int(func_args[0])
        
        if 'ABSTRACT 'in type_info:
            res={}
        
        
        return res
        
    def parseModule(self,info):
        res={}
        type_info = info[2]
        return res
        
    def parseDT(self,info):
        res={}
        res['arg']=[]
        
        e = split_brackets(info[1].strip())
        
        for i in e:
            #Remove whitespace and the first and last bracket
            i=i.strip()[1:-1]
            dtEl={}
            dtEl['num'], dtEl['name'] = i.split()[0:2]
            dtEl['name'] = dtEl['name'].lower().replace("'","")
            info_el = split_brackets(i[i.index("(")-1:],remove_b=False)
            #Re-order to be the same as everything else
            newL = [info_el[2],'()',info_el[0],'()',info_el[1]]
            dtEl['var'] = self.parseVar(newL)
            res['arg'].append(dtEl)
        
        return res
        
    def parseFuncArg(self,info):
        r=self.parseVar(info)
        return r
        
    def parseParam(self,info):
        res={}
        
        type_info = info[2]
        value = info[4]
        
        p, c, s=self.getVarType(type_info)
        
        res['pytype'] = p
        res['ctype'] = c
        res['bytes'] = s
        
        res['value']=self.getParamValue(value,res['pytype'])
        
        res['array']=False
        if 'ARRAY' in value:
            res['array']=True
        
        return res


    def getVarType(self,x):
        x=x.strip()
        if x.startswith('('):
            x=x[1:].strip()
       
        x = x.split()
        size = x[1]
        pytype, ctype = self.getTypes(x[0],size)
        
        return pytype, ctype, size
        
    def getParamValue(self,x,typ):
        if 'ARRAY' in x:
            x = split_brackets(x.strip())
            arrLen = int(''.join(c for c in x[-1] if c not in "'()"))
            x2 = split_brackets(x[1][x[1].index("("):])
            listParam = [y.split("'")[-2] for y in x2]
            p = [self.parseSingleParam(y,typ) for y in listParam]
        else:
            if 'COMPLEX' in x:
                yy = x.split("'")
                p = complex(self.parseSingleParam(yy[-4],'float'),
                     self.parseSingleParam(yy[-2],'float'))
            else:
                y = x.split()[-1][:-1]
                y = y.replace("'","")
                p = self.parseSingleParam(y,typ) 
            
        return p

    def parseSingleParam(self,x,typ):
        if '@' in x:
            return hextofloat(x)
        else:
            return getattr(__builtin__, typ)(x)
            
    def processArray(self,info):
        r = {}
        if 'ALLOCATABLE' in info[0]:
            r['atype'] = 'alloc'
        elif 'POINTER' in info[0]:
            r['atype'] = 'pointer'
        elif 'ASSUMED_SHAPE' in info[4]:
            r['atype'] = 'assumed_shape'
        elif 'ASSUMED_SIZE' in info[4]:
            r['atype'] = 'assumed_size'
        elif 'CONSTANT' in info[4]:
            r['bounds'] = self.getBounds(info)
            r['atype'] = 'explicit'
        r['ndims'] = self.getNdims(info)
        
        p, c, s=self.getVarType(info[2])
        
        r['pytype'] = p
        r['ctype'] = c
        r['bytes'] = s
        
        return r
        
    def parseDTType(self, info):
        #Nothing of any use in the info list
        return None
        
    def getNdims(self, info):
        return info[4].replace("(", "").strip().split()[0]
    
    def getBounds(self, info):
        try:
            # Horrible but easier than splitting the nested brackets
            return [int(x) for x in info[4].split("'")[1:-1:2]]
        except ValueError:
            # Sometimes we cant know the size till run time
            return -1
 
    def getStrLen(self, info):
        # Horrible but easier than splitting the nested brackets
        y=info[2].split("'")[1:-1:2]
        if len(y)==0:
            y=-1
        else:
            y=y[0]
        return int(y)
    
    def processDTVar(self,info):
        # size is actually the dt definition number
        p, c, s=self.getVarType(info[2])
        return self.matchDtDef(int(s))
    

    def getTypes(self,x,size):
        if 'CHARACTER' in x:
            pytype='str'
            ctype='c_char_p'
        elif 'INTEGER' in x:
            pytype='int'
            ctype=self.getCtypeIntSize(size)
        elif 'REAL' in x:
            pytype,ctype=self.getCtypeFloatSize(size)
        elif 'COMPLEX' in x:
            pytype,ctype=self.getCtypeFloatSize(size)
            pytype='complex'
        elif 'LOGICAL' in x:
            pytype='bool'
            ctype='c_bool'
        elif 'UNKNOWN' in x:
            pytype='None'
            ctype='c_void_p'
        elif "DERIVED" in x:
            pytype='dict'
            ctype='c_void_p'
        else:
            pytype='None'
            ctype='c_void_p'
            #raise ValueError("Cant parse " + x)
            
        if 'PASS ' in x:
            pytype='None'
            ctype='c_void_p'
            
        return pytype,ctype
        
    def getCtypeIntSize(self,size):
        size = int(size)
        if size == ctypes.sizeof(ctypes.c_int):
            res = 'c_int'
        elif size == ctypes.sizeof(ctypes.c_int16):
            res = 'c_int16'
        elif size == ctypes.sizeof(ctypes.c_int32):
            res = 'c_int32'
        elif size == ctypes.sizeof(ctypes.c_int64):
            res = 'c_int64'
        elif size == ctypes.sizeof(ctypes.c_byte):
            res = 'c_byte'
        elif size == ctypes.sizeof(ctypes.c_short):
            res = 'c_short'
        else:
            raise ValueError("Cant find suitable int for size " + str(size))
        return res
    
    def getCtypeFloatSize(self,size):
        size = int(size)
        pytype='float'
        if size == ctypes.sizeof(ctypes.c_float):
            res = 'c_float'
        elif size == ctypes.sizeof(ctypes.c_double):
            res = 'c_double'
        elif size == ctypes.sizeof(ctypes.c_long):
            res = 'c_long'
        elif size == ctypes.sizeof(ctypes.c_longdouble):
            res = 'c_longdouble'
            pytype='quad'
        elif size == ctypes.sizeof(ctypes.c_longlong):
            res = 'c_long'
            pytype='quad'
        else:
            raise ValueError("Cant find suitable float for size " + str(size))
        return pytype,res
    
    def parseIntent(self,info):
        value = False
        if ' INOUT ' in info:
            value = 'inout'
        elif ' OUT ' in info:
            value = 'out'
        elif ' IN ' in info:
            value = 'in'
        elif ' UNKNOWN-INTENT ' in info:
            value = 'na'
        return value

    def matchDtDef(self,num):
        for i in self.dt_defines:
            if int(num) == int(i['num']):
                return i
        
        print("Cant match dt definition "+str(num))
 
    def matchFuncArgs(self):
        ind_funcs=[]
        ind_func_args=[]
        
        func_nums=[]
        func_arg_num=[]
        
        for idx,i in enumerate(self.all_symbols):
            if 'proc' in i:
                ind_funcs.append(idx)
                func_nums.append(int(i['proc']['func_arg_id']))
            if 'func_arg' in i:
                ind_func_args.append(idx)
                func_arg_num.append(int(i['parent_id']))

        s_f = np.argsort(func_nums,kind='heapsort')

        sorted_index = np.searchsorted(func_nums,func_arg_num,sorter=s_f,side='left')
                
        lenfn=len(s_f)
        # Things with index greater than the length of the function nums are
        # arguments to functions that are argument to funcs
        sorted_index = [i if i<lenfn else None for i in sorted_index]

        for i,j in zip(sorted_index,ind_func_args):
            if i is not None:
                fAind=s_f[i]
                self.all_symbols[ind_funcs[fAind]]['arg'].append(self.all_symbols[j])
        









def split_brackets(value,remove_b=True):
    if isinstance(value,unicode):
        return split_brackets_c(str(value),remove_b)
    else:
        return split_brackets_c(value,remove_b)

cdef split_brackets_c(str value, bint remove_b=True):
    '''
    Split a string based on pairs of brackets, nested brackets are not split
    
    Input:
        'abc (def) (fgh () ())'
    
    Outputs:
        ['abc (def')', '(fgh () ())']
    '''
    cdef:
        int idx,j,j2,count,lvalue
        bint start
        str i
        list res
    
    if remove_b:
        if value.startswith('(') and value.endswith(')'):
            value = value[1:-1]
    
    res = []
    start = False
    count = 0
    j = 0
    for idx,i in enumerate(value):
        if i == '(':
            count = count + 1
            start = True
        if i == ')':
            count = count - 1
        if start:
            if count == 0:
                j2=idx+1
                res.append(value[j:j2])
                j=j2
                start = False
                
    if not j == len(value):
        res.append(value[j:])
    
    return res
    



import os, sys

os.environ["_GFORT2PY_TEST_FLAG"] = "1"

import numpy as np
import gfort2py as gf

try:
	import unittest as unittest
except ImportError:
	import unittest2 as unittest
	
import subprocess
import numpy.testing as np_test

from contextlib import contextmanager
try:
	from StringIO import StringIO
	from BytesIO import BytesIO
except ImportError:
	from io import StringIO
	from io import BytesIO

os.chdir('tests')
subprocess.check_output(["make"])
x=gf.fFort('./tester.so','tester.mod',rerun=True)

#Decreases recursion depth to make debugging easier
#sys.setrecursionlimit(100)


@contextmanager
def captured_output():
	"""
	For use when we need to grab the stdout/stderr from fortran (but only in testing)
	Use as:
	with captured_output() as (out,err):
		func()
	output=out.getvalue().strip()
	error=err.getvalue().strip()
	"""
	new_out, new_err = StringIO(),StringIO()
	old_out,old_err = sys.stdout, sys.stderr
	try:
		sys.stdout, sys.stderr = new_out, new_err
		yield sys.stdout, sys.stderr
	finally:
		sys.stdout, sys.stderr = old_out, old_err

class TestStringMethods(unittest.TestCase):
	
	def test_mising_var(self):	
		with self.assertRaises(AttributeError) as cm:
			a=x.invalid_var.get()
	
	def test_a_str(self):
		v='123456798'
		x.a_str=v
		self.assertEqual(x.a_str.get(),v)
		
	def test_a_str_bad_length(self):
		v='132456789kjhgjhf'
		x.a_str=v
		self.assertEqual(x.a_str.get(),v[0:10])
		
	def test_a_int(self):
		v=1
		x.a_int=v
		self.assertEqual(x.a_int.get(),v)
		
	def test_a_int_str(self):
		with self.assertRaises(ValueError) as cm:
			x.a_int='abc'
			
	def test_a_real(self):
		v=1.0
		x.a_real=v
		self.assertEqual(x.a_real.get(),v)
	
	def test_a_real_str(self):	
		with self.assertRaises(ValueError) as cm:
			x.a_real='abc'
			
	def test_const_int_set(self):	
		with self.assertRaises(ValueError) as cm:
			x.const_int=2
			
	def test_const_int(self):	
		self.assertEqual(x.const_int.get(),1)	

	def test_const_int_p1(self):	
		self.assertEqual(x.const_int_p1.get(),2)	

	def test_const_int_long(self):	
		self.assertEqual(x.const_int_lp.get(),1)	

	def test_const_real_dp(self):	
		self.assertEqual(x.const_real_dp.get(),1.0)
		
	def test_const_real_pi_dp(self):	
		self.assertEqual(x.const_real_pi_dp.get(),3.14)
		
	def test_const_real_qp(self):	
		self.assertEqual(x.const_real_qp.get(),1.0)

	def test_const_int_arr_error(self):	
		with self.assertRaises(ValueError) as cm:
			x.const_int_arr='abc'
		
	def test_const_int_arr(self):	
		np_test.assert_array_equal(x.const_int_arr.get(),np.array([1,2,3,4,5,6,7,8,9,0],dtype='int'))

	def test_const_real_arr(self):	
		np_test.assert_array_equal(x.const_real_arr.get(),np.array([1.0,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0,0.0],dtype='float'))

	def test_const_dp_arr(self):	
		np_test.assert_array_equal(x.const_real_dp_arr.get(),np.array([1.0,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0,0.0],dtype='float'))

	def test_b_int_exp_1d(self):
		v=np.random.randint(0,100,size=(5))
		x.b_int_exp_1d=v
		np_test.assert_array_equal(x.b_int_exp_1d.get(),v)
		
	def test_b_int_exp_2d(self):
		v=np.random.randint(0,100,size=(5,5))
		x.b_int_exp_2d=v
		np_test.assert_array_equal(x.b_int_exp_2d.get(),v)
		
	def test_b_int_exp_3d(self):
		v=np.random.randint(0,100,size=(5,5,5))
		x.b_int_exp_3d=v
		np_test.assert_array_equal(x.b_int_exp_3d.get(),v)
		
	def test_b_int_exp_4d(self):
		v=np.random.randint(0,100,size=(5,5,5,5))
		x.b_int_exp_4d=v
		np_test.assert_array_equal(x.b_int_exp_4d.get(),v)
		
	def test_b_int_exp_5d(self):
		v=np.random.randint(0,100,size=(5,5,5,5,5))
		x.b_int_exp_5d=v
		np_test.assert_array_equal(x.b_int_exp_5d.get(),v)
		
	def test_b_real_exp_1d(self):
		v=np.random.random(size=(5))
		x.b_real_exp_1d=v
		np_test.assert_allclose(x.b_real_exp_1d.get(),v)
		
	def test_b_real_exp_2d(self):
		v=np.random.random(size=(5,5))
		x.b_real_exp_2d=v
		np_test.assert_allclose(x.b_real_exp_2d.get(),v)
		
	def test_b_real_exp_3d(self):
		v=np.random.random(size=(5,5,5))
		x.b_real_exp_3d=v
		np_test.assert_allclose(x.b_real_exp_3d.get(),v)
		
	def test_b_real_exp_4d(self):
		v=np.random.random(size=(5,5,5,5))
		x.b_real_exp_4d=v
		np_test.assert_allclose(x.b_real_exp_4d.get(),v)
		
	def test_b_real_exp_5d(self):
		v=np.random.random(size=(5,5,5,5,5))
		x.b_real_exp_5d=v
		np_test.assert_allclose(x.b_real_exp_5d.get(),v)
		
	def test_b_real_dp_exp_1d(self):
		v=np.random.random(size=(5))
		x.b_real_dp_exp_1d=v
		np_test.assert_allclose(x.b_real_dp_exp_1d.get(),v)
		
	def test_b_real_dp_exp_2d(self):
		v=np.random.random(size=(5,5))
		x.b_real_dp_exp_2d=v
		np_test.assert_allclose(x.b_real_dp_exp_2d.get(),v)
		
	def test_b_real_dp_exp_3d(self):
		v=np.random.random(size=(5,5,5))
		x.b_real_dp_exp_3d=v
		np_test.assert_allclose(x.b_real_dp_exp_3d.get(),v)
		
	def test_b_real_dp_exp_4d(self):
		v=np.random.random(size=(5,5,5,5))
		x.b_real_dp_exp_4d=v
		np_test.assert_allclose(x.b_real_dp_exp_4d.get(),v)
		
	def test_b_real_dp_exp_5d(self):
		v=np.random.random(size=(5,5,5,5,5))
		x.b_real_dp_exp_5d=v
		np_test.assert_allclose(x.b_real_dp_exp_5d.get(),v)

	def test_a_int_point(self):
		v=1
		x.a_int_point=v
		self.assertEqual(x.a_int_point.get(),v)

	def test_a_int_lp_point(self):
		v=1
		x.a_int_lp_point=v
		self.assertEqual(x.a_int_lp_point.get(),v)

	def test_a_real_point(self):
		v=1.0
		x.a_real_point=v
		self.assertEqual(x.a_real_point.get(),v)
		
	def test_a_real_dp_point(self):
		v=1.0
		x.a_real_dp_point=v
		self.assertEqual(x.a_real_dp_point.get(),v)
		
	def test_a_real_qp_point(self):
		v=1.0
		x.a_real_qp_point=v
		self.assertEqual(x.a_real_qp_point.get(),v)
		
	def test_a_str_point(self):
		v='abcdefghij'
		x.a_str_point=v
		self.assertEqual(x.a_str_point.get(),v)

	def test_a_int_target(self):
		v=1
		x.a_int_target=v
		self.assertEqual(x.a_int_target.get(),v)

	def test_a_int_lp_target(self):
		v=1
		x.a_int_lp_target=v
		self.assertEqual(x.a_int_lp_target.get(),v)

	def test_a_real_target(self):
		v=1.0
		x.a_real_target=v
		self.assertEqual(x.a_real_target.get(),v)
		
	def test_a_real_dp_target(self):
		v=1.0
		x.a_real_dp_target=v
		self.assertEqual(x.a_real_dp_target.get(),v)
		
	def test_a_real_qp_target(self):
		v=1.0
		x.a_real_qp_target=v
		self.assertEqual(x.a_real_qp_target.get(),v)
		
	def test_a_str_target(self):
		v='abcdefghij'
		x.a_str_target=v
		self.assertEqual(x.a_str_target.get(),v)

	def test_a_const_cmplx(self):
		self.assertEqual(x.const_cmplx.get(),complex(1.0,1.0))
		
	def test_a_const_cmplx_dp(self):
		self.assertEqual(x.const_cmplx_dp.get(),complex(1.0,1.0))
		
	def test_a_const_cmplx_qp(self):
		self.assertEqual(x.const_cmplx_qp.get(),complex(1.0,1.0))
		
	def test_a_cmplx(self):
		v=complex(1.0,1.0)
		x.a_cmplx=v
		self.assertEqual(x.a_cmplx.get(),v)

	def test_a_cmplx_dp(self):
		v=complex(1.0,1.0)
		x.a_cmplx_dp=v
		self.assertEqual(x.a_cmplx_dp.get(),v)
		
	def test_a_cmplx_qp(self):
		v=complex(1.0,1.0)
		x.a_cmplx_qp=v
		self.assertEqual(x.a_cmplx_qp.get(),v)
		
	def test_sub_no_args(self):
		with captured_output() as (out,err):
			x.sub_no_args()
		output=out.getvalue().strip()
		self.assertEqual(output,"1")
		
	def test_sub_alter_mod(self):
		y=x.sub_alter_mod()
		self.assertEqual(x.a_int.get(),99)
		self.assertEqual(x.a_int_lp.get(),99)
		self.assertEqual(x.a_real.get(),99.0)
		self.assertEqual(x.a_real_dp.get(),99.0)
		#self.assertEqual(x.a_real_qp.get(),99.0)
		self.assertEqual(x.a_str.get(),"9999999999")
		self.assertEqual(x.a_cmplx.get(),complex(99.0,99.0))
		self.assertEqual(x.a_cmplx_dp.get(),complex(99.0,99.0))
		#self.assertEqual(x.a_cmplx_qp.get(),complex(99.0,99.0))	
		
	def test_sub_alloc_1d_arrs(self):
		y=x.sub_alloc_int_1d_arrs()

	def test_func_int_in(self):
		v=5
		y=x.func_int_in(v)
		self.assertEqual(int(y),2*v)
		
	def test_func_int_in_multi(self):
		v=5
		w=3
		u=4
		y=x.func_int_in_multi(v,w,u)
		self.assertEqual(y,v+w+u)
		
	def test_sub_int_in(self):
		v=5
		with captured_output() as (out,err):
			y=x.sub_int_in(v)
		output=out.getvalue().strip()
		self.assertEqual(int(output),2*v)	

	def test_func_int_no_args(self):
		y=x.func_int_no_args()
		self.assertEqual(y,2)
		
	def test_func_real_no_args(self):
		y=x.func_real_no_args()
		self.assertEqual(y,3.0)

	def test_func_real_dp_no_args(self):
		y=x.func_real_dp_no_args()
		self.assertEqual(y,4.0)
		
	def test_sub_str_in_explicit(self):
		v='1324567980'
		with captured_output() as (out,err):
			y=x.sub_str_in_explicit(v)
		output=out.getvalue().strip()
		self.assertEqual(output,v)	
		
	def test_sub_str_in_implicit(self):
		v='123456789'
		with captured_output() as (out,err):
			y=x.sub_str_in_implicit(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,v)	
	
	def test_sub_str_multi(self):
		v=5
		u='123456789'
		w=4
		with captured_output() as (out,err):
			y=x.sub_str_multi(v,u,w)
		output=out.getvalue().strip()	
		self.assertEqual(output,str(v+w)+' '+u)	
		
		
	def test_sub_exp_array_int_1d(self):
		v=np.arange(0,5)
		o=' '.join([str(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_int_1d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())	
		
	def test_sub_exp_array_int_2d(self):
		v=np.arange(0,5*5).reshape((5,5))
		o=''.join([str(i).zfill(2).ljust(3) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_int_2d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())	

	def test_sub_exp_array_int_3d(self):
		v=np.arange(0,5*5*5).reshape((5,5,5))
		o=''.join([str(i).zfill(3).ljust(4) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_int_3d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())			


	def test_sub_exp_array_real_1d(self):
		v=np.arange(0,5.0).reshape((5))
		o='  '.join(["{:>4.1f}".format(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_real_1d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())	
		
	def test_sub_exp_array_real_2d(self):
		v=np.arange(0,5.0*5.0).reshape((5,5))
		o='  '.join(["{:>4.1f}".format(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_real_2d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())	

	def test_sub_exp_array_real_3d(self):
		v=np.arange(0,5.0*5.0*5.0).reshape((5,5,5))
		o=' '.join(["{:>5.1f}".format(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_real_3d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())	 

	def test_sub_exp_array_int_1d_multi(self):
		u=19
		w=20
		v=np.arange(0,5)
		o=' '.join([str(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_int_1d_multi(u,v,w)
		output=out.getvalue().strip()	
		self.assertEqual(output,str(u)+' '+o.strip()+' '+str(w)) 
 
 
	def test_sub_exp_array_real_dp_1d(self):
		v=np.arange(0,5.0).reshape((5))
		o='  '.join(["{:>4.1f}".format(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_real_dp_1d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())	
		
	def test_sub_exp_array_real_dp_2d(self):
		v=np.arange(0,5.0*5.0).reshape((5,5))
		o='  '.join(["{:>4.1f}".format(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_real_dp_2d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())	

	def test_sub_exp_array_real_dp_3d(self):
		v=np.arange(0,5.0*5.0*5.0).reshape((5,5,5))
		o=' '.join(["{:>5.1f}".format(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_real_dp_3d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())	   

	def test_sub_int_out(self):
		v=5
		with captured_output() as (out,err):
			y=x.sub_int_out(v)
		output=out.getvalue().strip()
		self.assertEqual(y,{'x':1})		

	def test_sub_int_inout(self):
		v=5
		with captured_output() as (out,err):
			y=x.sub_int_inout(v)
		output=out.getvalue().strip()
		self.assertEqual(y,{'x':2*v})
		
	def test_sub_int_no_intent(self):
		v=5
		with captured_output() as (out,err):
			y=x.sub_int_no_intent(v)
		output=out.getvalue().strip()
		self.assertEqual(y,{'x':2*v})
		
	def test_sub_real_inout(self):
		v=5.0
		with captured_output() as (out,err):
			y=x.sub_real_inout(v)
		output=out.getvalue().strip()
		self.assertEqual(y,{'x':2*v})
		
	def test_sub_exp_inout(self):
		v=np.array([1,2,3,4,5])
		with captured_output() as (out,err):
			y=x.sub_exp_inout(v)
		output=out.getvalue().strip()

		np_test.assert_array_equal(y['x'],2*v)
		
	def test_dt_set_value(self):
		x.f_struct_simple.x=1
		x.f_struct_simple.y=0
		y=x.f_struct_simple.get()
		self.assertEqual(y,{'x':1,'y':0})
		
	def test_dt_set_dict(self):	
		x.f_struct_simple={'x':5,'y':5}
		y=x.f_struct_simple.get()
		self.assertEqual(y,{'x':5,'y':5})
			
	def test_dt_bad_dict(self):
		with self.assertRaises(ValueError) as cm:
			x.f_struct_simple = {'asw':2,'y':0}
			
	def test_dt_bad_value(self):
		with self.assertRaises(TypeError) as cm:
			x.f_struct_simple.x='asde'
	
	#def test_c_int_alloc_1d_non_alloc(self):
		#y=x.sub_alloc_int_1d_cleanup()
		#with self.assertRaises(ValueError) as cm:
			#x.c_int_alloc_1d.get()
			
	def test_c_int_alloc_1d(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5])
		v[:]=1
		np_test.assert_array_equal(x.c_int_alloc_1d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_2d(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_int_alloc_2d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_3d(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_int_alloc_3d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_4d(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5,5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_int_alloc_4d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_5d(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5,5,5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_int_alloc_5d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
		
	
	def test_c_int_alloc_1d_set(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5])
		v[:]=5
		x.c_int_alloc_1d = v
		np_test.assert_array_equal(x.c_int_alloc_1d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_2d_set(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5,5])
		v[:]=5
		x.c_int_alloc_2d = v
		np_test.assert_array_equal(x.c_int_alloc_2d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_3d_set(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5,5,5])
		v[:]=5
		x.c_int_alloc_3d = v
		np_test.assert_array_equal(x.c_int_alloc_3d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_4d_set(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5,5,5,5])
		v[:]=5
		x.c_int_alloc_4d = v
		np_test.assert_array_equal(x.c_int_alloc_4d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_5d_set(self):
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([5,5,5,5,5])
		v[:]=5
		x.c_int_alloc_5d = v
		np_test.assert_array_equal(x.c_int_alloc_5d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()
	
	def test_c_int_alloc_1d_large(self):
		# Can have issues exiting when using large (>255) arrays
		y=x.sub_alloc_int_1d_cleanup()
		y=x.sub_alloc_int_1d_arrs()
		v=np.zeros([256])
		v[:]=5
		x.c_int_alloc_1d = v
		np_test.assert_array_equal(x.c_int_alloc_1d.get(),v)
		y=x.sub_alloc_int_1d_cleanup()


	def test_c_real_alloc_1d(self):
		y=x.sub_alloc_real_1d_cleanup()
		y=x.sub_alloc_real_1d_arrs()
		v=np.zeros([5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_alloc_1d.get(),v)
		y=x.sub_alloc_real_1d_cleanup()
	
	def test_c_real_alloc_2d(self):
		y=x.sub_alloc_real_1d_cleanup()
		y=x.sub_alloc_real_1d_arrs()
		v=np.zeros([5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_alloc_2d.get(),v)
		y=x.sub_alloc_real_1d_cleanup()
	
	def test_c_real_alloc_3d(self):
		y=x.sub_alloc_real_1d_cleanup()
		y=x.sub_alloc_real_1d_arrs()
		v=np.zeros([5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_alloc_3d.get(),v)
		y=x.sub_alloc_real_1d_cleanup()
	
	def test_c_real_alloc_4d(self):
		y=x.sub_alloc_real_1d_cleanup()
		y=x.sub_alloc_real_1d_arrs()
		v=np.zeros([5,5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_alloc_4d.get(),v)
		y=x.sub_alloc_real_1d_cleanup()
	
	def test_c_real_alloc_5d(self):
		y=x.sub_alloc_real_1d_cleanup()
		y=x.sub_alloc_real_1d_arrs()
		v=np.zeros([5,5,5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_alloc_5d.get(),v)
		y=x.sub_alloc_real_1d_cleanup()
		
	
	def test_c_real_dp_alloc_1d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5])
		v[:]=2.0
		x.c_real_dp_alloc_1d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_1d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_2d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5])
		v[:]=2.0
		x.c_real_dp_alloc_2d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_2d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_3d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5])
		v[:]=2.0
		x.c_real_dp_alloc_3d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_3d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_4d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5,5])
		v[:]=2.0
		x.c_real_dp_alloc_4d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_4d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_5d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5,5,5])
		v[:]=2.0
		x.c_real_dp_alloc_5d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_5d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	

	def test_c_real_dp_alloc_1d(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_dp_alloc_1d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_2d(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_dp_alloc_2d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_3d(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_dp_alloc_3d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_4d(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_dp_alloc_4d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_5d(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5,5,5])
		v[:]=1
		np_test.assert_array_equal(x.c_real_dp_alloc_5d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
		
	
	def test_c_real_dp_alloc_1d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5])
		v[:]=2.0
		x.c_real_dp_alloc_1d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_1d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_2d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5])
		v[:]=2.0
		x.c_real_dp_alloc_2d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_2d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_3d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5])
		v[:]=2.0
		x.c_real_dp_alloc_3d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_3d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_4d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5,5])
		v[:]=2.0
		x.c_real_dp_alloc_4d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_4d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
	
	def test_c_real_dp_alloc_5d_set(self):
		y=x.sub_alloc_real_dp_1d_cleanup()
		y=x.sub_alloc_real_dp_1d_arrs()
		v=np.zeros([5,5,5,5,5])
		v[:]=2.0
		x.c_real_dp_alloc_5d = v
		np_test.assert_array_equal(x.c_real_dp_alloc_5d.get(),v)
		y=x.sub_alloc_real_dp_1d_cleanup()
		
	def test_func_return_res(self):
		y=x.func_return_res(2)
		self.assertEqual(y,True)
		y=x.func_return_res(10)
		self.assertEqual(y,False)		
	

	def test_func_assumed_shape_arr_1d(self):
		v=np.zeros([5],dtype='int32')
		v[0]=2.0
		y=x.func_assumed_shape_arr_1d(v)
		self.assertEqual(y,True)
		
	def test_func_assumed_shape_arr_2d(self):
		v=np.zeros([5,5],dtype='int32')
		v[1,0]=2.0
		y=x.func_assumed_shape_arr_2d(v)
		self.assertEqual(y,True)
		
	def test_func_assumed_shape_arr_3d(self):
		v=np.zeros([5,5,5],dtype='int32')
		v[2,1,0]=2.0
		y=x.func_assumed_shape_arr_3d(v)
		self.assertEqual(y,True)
		
	def test_func_assumed_shape_arr_4d(self):
		v=np.zeros([5,5,5,5],dtype='int32')
		v[3,2,1,0]=2.0
		y=x.func_assumed_shape_arr_4d(v)
		self.assertEqual(y,True)
		
	def test_func_assumed_shape_arr_5d(self):
		v=np.zeros([5,5,5,5,5],dtype='int32')
		v[4,3,2,1,0]=2.0
		y=x.func_assumed_shape_arr_5d(v)
		self.assertEqual(y,True)
		
		
	def test_func_assumed_size_arr_1d(self):
		v=np.zeros([5],dtype='int32')
		v[1]=2
		y=x.func_assumed_size_arr_1d(v)
		self.assertEqual(y,True)
		
	def test_func_assumed_size_arr_real_1d(self):
		v=np.zeros([5],dtype='float32')
		v[1]=2.0
		y=x.func_assumed_size_arr_real_1d(v)
		self.assertEqual(y,True)
		
	def test_func_assumed_size_arr_real_dp_1d(self):
		v=np.zeros([5],dtype='float64')
		v[1]=2.0
		y=x.func_assumed_size_arr_real_dp_1d(v)
		self.assertEqual(y,True)
		
	def test_sub_alloc_arr_1d(self):
		v=0
		y=x.sub_alloc_arr_1d(v)
		vTest=np.zeros(10)
		vTest[:]=10
		np_test.assert_array_equal(y['x'],vTest)
		
	def test_sub_dt_in_s_simple(self):
		with captured_output() as (out,err):
			y=x.sub_f_simple_in({'x':1,'y':10})
		output=out.getvalue().strip()
		o=' '.join([str(i) for i in [1,10]])
		self.assertEqual(output,o)
	
	def test_sub_dt_out_s_simple(self):
		with captured_output() as (out,err):
			y=x.sub_f_simple_out({})
		output=out.getvalue().strip()
		self.assertEqual(y['x'],{'x':1,'y':10})	
	
	def test_sub_dt_inout_s_simple(self):
		with captured_output() as (out,err):
			y=x.sub_f_simple_inout({'x':5,'y':3})
		output=out.getvalue().strip()
		o='  '.join([str(i) for i in [5,3]])
		self.assertEqual(output,o)
		self.assertEqual(y['zzz'],{'x':1,'y':10})
		
	def test_sub_dt_inoutp_s_simple(self):
		with captured_output() as (out,err):
			y=x.sub_f_simple_inoutp({'x':5,'y':3})
		output=out.getvalue().strip()
		o='  '.join([str(i) for i in [5,3]])
		self.assertEqual(output,o)
		self.assertEqual(y['zzz'],{'x':1,'y':10})
		
	def test_sub_int_p(self):
		with captured_output() as (out,err):
			y=x.sub_int_p(1)
		output=out.getvalue().strip()
		self.assertEqual(output,'1')
		self.assertEqual(y['zzz'],5)

	def test_sub_real_p(self):
		with captured_output() as (out,err):
			y=x.sub_real_p(1.0)
		output=out.getvalue().strip()
		self.assertEqual(output,'1.00')
		self.assertEqual(y['zzz'],5.0)
		
	def test_sub_str_p(self):
		with captured_output() as (out,err):
			y=x.sub_str_p('abcdef')
		output=out.getvalue().strip()
		self.assertEqual(output,'abcdef')
		self.assertEqual(y['zzz'],'xyzxyz')
		
	def test_sub_arr_exp_p(self):
		v=np.arange(0,5)
		o=' '.join([str(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_exp_array_int_1d(v)
		output=out.getvalue().strip()	
		self.assertEqual(output,o.strip())
	
	#@unittest.skip("Skipping")	
	def test_sub_arr_assumed_rank_int_1d(self):
		v=np.arange(10.0,15.0)
		o=' '.join([str(i) for i in v.flatten()])
		with captured_output() as (out,err):
			y=x.sub_arr_assumed_rank_int_1d(v)
		output=out.getvalue().strip()	
		np_test.assert_array_equal(y['zzz'],np.array([100.0]*5))
	
	def test_sub_opt(self):
		with captured_output() as (out,err):
			y=x.sub_int_opt(1)
		output=out.getvalue().strip()
		self.assertEqual(output,'100')
		with captured_output() as (out,err):
			y=x.sub_int_opt()
		output=out.getvalue().strip()
		self.assertEqual(output,'200')
	
	def test_dt_copy(self):
		x.f_struct_simple.x=99
		x.f_struct_simple.y=99
		y=x.f_struct_simple.get()
		self.assertEqual(y,{'x':99,'y':99})
		y=x.f_struct_simple.get(copy=True)
		self.assertEqual(y,{'x':99,'y':99})
		y=x.f_struct_simple.get(copy=False)
		self.assertEqual(y.x,99)
		self.assertEqual(y.y,99)
		
		
	def test_second_mod(self):
		x.f_struct_simple2.x=99
		y=x.sub_use_mod()
		self.assertEqual(x.test2_x.get(),1)
		self.assertEqual(x.f_struct_simple2.get(),{'x':5,'y':6})
	
	def test_nested_dts(self):
		x.g_struct.a_int=10
		self.assertEqual(x.g_struct.a_int,10)
		x.g_struct={'a_int':10,'f_struct':{'a_int':3}}
		self.assertEqual(x.g_struct.f_struct.a_int,3)
		x.g_struct.f_struct.a_int=8
		self.assertEqual(x.g_struct.f_struct.a_int,8)
		y=x.func_check_nested_dt()
		self.assertEqual(y,True)
	
	def test_logical_arr(self):
		xarr=np.zeros(10)
		x2arr=np.zeros(10)
		x2arr[:]=False
		xarr[:]=True
		
		y=x.func_alltrue_arr_1d(xarr)
		y2=x.func_allfalse_arr_1d(x2arr)
		self.assertEqual(y,True)
		self.assertEqual(y2,True)


if __name__ == '__main__':
	unittest.main() 






from __future__ import print_function

def find_key_val(list_dicts, key, value):
    v = value.lower()
    for idx, i in enumerate(list_dicts):
        if i[key].lower() == v:
            return idx




from __future__ import print_function
try:
    import __builtin__
except ImportError:
    import builtins as __builtin__

import ctypes
import os
import six
import select

from .var import fVar
from .cmplx import fComplex
from .arrays import fExplicitArray, fDummyArray, fAssumedShape, fAssumedSize, fAllocatableArray
from .strings import fStr
from .types import fDerivedType
from .utils import *
from .errors import *


_TEST_FLAG = os.environ.get("_GFORT2PY_TEST_FLAG") is not None



class captureStdOut():
    def read_pipe(self,pipe_out):
        def more_data():
            r, _, _ = select.select([pipe_out], [], [], 0)
            return bool(r)
        out = b''
        while more_data():
            out += os.read(pipe_out, 1024)
        return out.decode()
    
    def __enter__(self):
        if _TEST_FLAG:
            self.pipe_out, self.pipe_in = os.pipe()
            self.stdout = os.dup(1)
            os.dup2(self.pipe_in, 1)
    
    def __exit__(self,*args,**kwargs):
        if _TEST_FLAG:
            os.dup2(self.stdout, 1)
            print(self.read_pipe(self.pipe_out))
            os.close(self.pipe_in)
            os.close(self.pipe_out)
            os.close(self.stdout)

class fFunc(fVar):

    def __init__(self, lib, obj):
        self.__dict__.update(obj)
        self._lib = lib
        self._sub = self.proc['sub']
        try:
            self._call = getattr(self._lib, self.mangled_name)
        except AttributeError:
            print("Skipping "+self.mangled_name)
            return
            
        self._set_return()
        self._set_arg_ctypes()
        self.save_args=False
        self.args_out = None

    def _set_arg_ctypes(self):
        self._arg_ctypes = []
        self._arg_vars = []
        
        tmp=[]
        if len(self.proc['arg_nums'])>0:
            for i in self.arg:
                self._arg_vars.append(self._init_var(i))
                self._arg_vars[-1]._func_arg=True
                
                if 'pointer' in i['var']:
                    pointer=True
                else:
                    pointer=False
                x,y=self._arg_vars[-1].ctype_def_func(pointer=pointer,intent=i['var']['intent'])
                self._arg_ctypes.append(x)
                if y is not None:
                    tmp.append(y)
                    
            self._call.argtypes = self._arg_ctypes+tmp
            

    def _init_var(self, obj):
        array = None
        if 'array' in obj['var']:
            array = obj['var']['array']
        
        if obj['var']['pytype'] == 'str':
            x = fStr(self._lib, obj)
        elif obj['var']['pytype'] == 'complex':
            x = fComplex(self._lib, obj)
        elif 'dt' in obj['var']:
            x = fDerivedType(self._lib, obj)
        elif array is not None:
            #print(self.name,array)
            if array['atype'] == 'explicit':
                x = fExplicitArray(self._lib, obj)
            elif array['atype'] == 'alloc':
                x = fAllocatableArray(self._lib, obj)
            elif array['atype'] == 'assumed_shape' or array['atype'] == 'pointer':
                x = fAssumedShape(self._lib, obj)
            elif array['atype'] == 'assumed_size':
                x = fAssumedSize(self._lib, obj)
            else:
                raise ValueError("Unknown array: "+str(obj))
        else:
            x = fVar(self._lib, obj)

        x._func_arg=True

        return x

    def _set_return(self):
        if not self._sub:
            self._restype = self.ctype_def()
            self._call.restype = self._restype
            
    def _args_to_ctypes(self,args):
        tmp = []
        args_in = []
        for vout, vin, fctype, a in six.moves.zip_longest(self._arg_vars, args, self._arg_ctypes, self.arg):
            if 'optional' in a['var'] and vin is None:
                #Missing optional arguments 
                args_in.append(None)            
            else:
                x,y=vout.py_to_ctype_f(vin)
                if 'pointer' in a['var']:
                    args_in.append(vout.py_to_ctype_p(vin))
                else:
                    args_in.append(x)
                if y is not None:
                    tmp.append(y)
                
        return args_in + tmp
        
    def ctype_def(self):
        """
        The ctype type of this object
        """
        if '_cached_ctype' not in self.__dict__:
            self._cached_ctype = getattr(ctypes, self.proc['ret']['ctype'])
        
        return self._cached_ctype
    
    def _ctypes_to_return(self,args_out):
    
        r = {}
        self.args_out = {}
        
        if self.save_args:
            # Save arguments inside this object
            for i,j in zip(self._arg_vars,args_out):
                if 'out' in i.var['intent'] or i.var['intent']=='na': 
                    r[i.name]=''
                    if hasattr(j,'contents'):
                        self.args_out[i.name]=j.contents
                    else:
                        self.args_out[i.name]=j
        else:
            # Copy arguments into a dict for returning
            for i,j in zip(self._arg_vars,args_out):
                if 'out' in i.var['intent'] or i.var['intent']=='na':
                    if hasattr(j,'contents'):
                        r[i.name]=i.ctype_to_py_f(j.contents)
                    else:
                        r[i.name]=i.ctype_to_py_f(j)

        return r
    
    def __call__(self, *args):
        args_in = self._args_to_ctypes(args)
        
        # Capture stdout messages
        with captureStdOut() as cs:        
            if len(args_in) > 0:
                res = self._call(*args_in)
            else:
                res = self._call()

        if self._sub:
            return self._ctypes_to_return(args_in)
        else:
            return self.returnPytype()(res)
            
    def saveArgs(self,v=False):
        """ Instead of copying arguments back we save them
        inside the func object so we dont need to copy them
        """
        if v:
            self.save_args=True
        else:
            self.save_args=False
            
    def returnPytype(self):
        if '_cached_pytype' not in self.__dict__:
            self._cached_pytype = getattr(__builtin__, self.proc['ret']['pytype'])
        
        return self._cached_pytype
            
    def __str__(self):
        return str("Function: " + self.name)

    def __repr__(self):
        return self.__str__()

    @property
    def __doc__(self):
        s = "Function: " + self.name + "("
        if len(self._arg_vars) > 0:
            s = s + ",".join([i._pname() for i in self._arg_vars])
        else:
            s = s + "None"
        s = s + ")" + os.linesep+' '
        s = s + "Args In: " + \
            ", ".join([i._pname()
                      for i in self._arg_vars if 'in' in i.var['intent']]) + ' '+os.linesep+' '
        s = s + "Args Out: " + \
            ", ".join([i._pname()
                      for i in self._arg_vars if 'out' in i.var['intent']]) + ' '+os.linesep+' '
        s = s + "Returns: "
        if self.sub:
            s = s + "None"
        else:
            s = s + str(self.pytype)
        s = s + os.linesep+' '
        return s

    def __bool__(self):
        return True
        
    def __len__(self):
        return 1



from __future__ import print_function
import ctypes
import functools
from .var import fVar
from .cmplx import fComplex
from .arrays import fExplicitArray, fDummyArray, fAssumedShape, fAssumedSize, fAllocatableArray
from .strings import fStr
from .errors import *


_dictAllDtDescs={}


def getEmptyDT(name):
    class emptyDT(ctypes.Structure):
        pass
    emptyDT.__name__ = name
    return emptyDT
        
class _DTDesc(object):
    def __init__(self,dt_def):
        self._lib = None
        self.dt_def = dt_def['dt_def']['arg']
        self.dt_name = dt_def['name'].lower().replace("'","")
        
        self.names = [i['name'].lower().replace("'","") for i in self.dt_def]
        self.args = []
        for i in self.dt_def:
            self.args.append(self._init_var(i))
            
        self.ctypes = []
        for i in self.args:
            try:
                self.ctypes.append(i.ctype_def())
            except AttributeError:
                if hasattr(i,'dt_desc'):
                    self.ctypes.append(i.dt_desc)
                else:
                    self.ctypes.append(ctypes.POINTER(getEmptyDT(self.dt_name)))
                
        self.dt_desc = self._create_dt()
             
    def _create_dt(self):
        class fDerivedTypeDesc(ctypes.Structure):
            _fields_ = list(zip(self.names,self.ctypes))
        fDerivedTypeDesc.__name__ = self.dt_name
        return fDerivedTypeDesc
        
    def _init_var(self, obj):
        # Placeholder for a dt
        if 'dt' in obj['var']:
            name = obj['var']['dt']['name'].lower().replace("'","")
            #By the time we get here we should have allready filled _dictAllDtDescs
            # with all the dt defs
            return _dictAllDtDescs[name]
        
        array = None
        if 'array' in obj['var']:
            array = obj['var']['array']
        
        pytype = obj['var']['pytype'] 
        
        if pytype in 'str':
            return fStr(self._lib, obj)
        elif pytype in 'complex':
            return fComplex(self._lib, obj)
        elif array is not None:
            atype = array['atype']
            if atype in 'explicit':
                return fExplicitArray(self._lib, obj)
            elif atype in 'alloc':
               return fAllocatableArray(self._lib, obj)
            elif atype in 'assumed_shape' or atype in 'pointer':
                return fAssumedShape(self._lib, obj)
            elif atype in 'assumed_size':
                return fAssumedSize(self._lib, obj)
            else:
                raise ValueError("Unknown array: "+str(obj))
        else:
           return fVar(self._lib, obj)



class fDerivedType(fVar):
    def __init__(self, lib, obj):
        self.__dict__.update(obj)
        self._lib = lib
        self._dt_type = self.var['dt']['name'].lower().replace("'","")

        self._dt_desc = _dictAllDtDescs[self._dt_type]
        self._desc = self._dt_desc.dt_desc
        self._ctype = self._desc
        self._ctype_desc = ctypes.POINTER(self._ctype)


        self._args = self._dt_desc.args
        self._nameArgs = self._dt_desc.names
        self._typeArgs = self._dt_desc.ctypes

        self.intent=None
        self.pointer=None
        
        #Store the ref to the lib object
        try:   
            self._ref = self._get_from_lib()
        except NotInLib:
            self._ref = None
        
        self._value = {}
        
    def get(self,copy=True):
        res={}
        if copy:
            for name,i in zip(self._nameArgs,self._args):
                x=getattr(self._ref,name)
                res[name]=i.ctype_to_py_f(x)
        else:
            if hasattr(self._ref,'contents'):
                res =self._ref.contents
            else:
                res = self._ref
        return res
            
    def set_mod(self,value):
        # Wants a dict
        if not all(i in self._nameArgs for i in value.keys()):
            raise ValueError("Dict contains elements not in struct")
        
        for name in value:
            self.set_single(name,value[name])
            
    def set_single(self,name,value):
        self._setSingle(self._ref,name,value)
        
    def _setSingle(self,v,name,value):
        if isinstance(value,dict):
            for i in value:
                self._setSingle(getattr(v,name),i,value[i])
        else:
            setattr(v,name,value)

    def py_to_ctype(self, value):
        """
        Pass in a python value returns the ctype representation of it
        """
        self._value=self._ctype()

        # Wants a dict
        if isinstance(value,dict):
            if not all(i in self._nameArgs for i in value.keys()):
                raise ValueError("Dict contains elements not in struct")
            
            for name in value.keys():
                setattr(self._value,name,value[name])
        
        
        return self._value
        
    def py_to_ctype_f(self, value):
        """
        Pass in a python value returns the ctype representation of it, 
        suitable for a function
        
        Second return value is anythng that needs to go at the end of the
        arg list, like a string len
        """
        r=self.py_to_ctype(value)    
            
        return r,None

    def ctype_to_py(self, value):
        """
        Pass in a ctype value returns the python representation of it
        """
        res={}
        for name,i in zip(self._nameArgs,self._args):
            x=getattr(value,name)
            res[name]=i.ctype_to_py_f(x)

        return res
        
    def ctype_to_py_f(self, value):
        """
        Pass in a ctype value returns the python representation of it,
        as returned by a function (may be a pointer)
        """
        return self.ctype_to_py(value)

    def ctype_def(self):
        """
        The ctype type of this object
        """
        return self._ctype

    def ctype_def_func(self,pointer=False,intent=''):
        """
        The ctype type of a value suitable for use as an argument of a function

        May just call ctype_def
        
        Second return value is anything that needs to go at the end of the
        arg list, like a string len
        """
        self.intent=intent
        self.pointer=pointer
        if pointer and intent is not 'na':
            f=ctypes.POINTER(self._ctype_desc)
        elif intent=='na':
            f=ctypes.POINTER(self._ctype_desc)
        else:
            f=self._ctype_desc
            
        return f,None
        
    def py_to_ctype_p(self,value):
        """
        The ctype representation suitable for function arguments wanting a pointer
        """
        return ctypes.POINTER(self.ctype_def())(self.py_to_ctype(value))
        
    def _pname(self):
        return str(self.name) + " <" + str(self._dt_def['name']) + ">"

    def __dir__(self):
        return self._nameArgs

    def __str__(self):
        return self.name+" <"+self._dt_type+" dt>"
        
    def __repr__(self):
        return self.name+" <"+self._dt_type+" dt>"
        
    def __getattr__(self, name): 
        if name in self.__dict__:
            return self.__dict__[name]

        if '_args' in self.__dict__ and '_nameArgs' in self.__dict__:
            if name in self._nameArgs:
                return self.__getitem__(name)

    def __setattr__(self, name, value):
        if '_nameArgs' in self.__dict__:
            if name in self._nameArgs:
                self.set_single(name,value)
                return
        
        self.__dict__[name] = value
        return    
        
    def get_dict(self):
        """
        Return a dict with the keys set suitable for this dt
        """
        x={}
        for i in self._nameArgs:
            x[i]=0
        return x
        
    def __getitem__(self,name=None):
        if name is None:
            raise KeyError
        if name not in self._nameArgs:
            raise KeyError("Name not in struct")
        
        if self._value is None or len(self._value)==0:
            return getattr(self._ref,name)
        else:
            return getattr(self._value,name)
        



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

from distutils.core import setup, Extension
from Cython.Distutils import build_ext
from Cython.Build import cythonize

import numpy as np
import os
import sysconfig

PY_INCLUDE = sysconfig.get_paths()['include']

ext = [
Extension("gfort2py.parseMod.utils_cpython",["gfort2py/parseMod/utils_cpython.pyx"],include_dirs=[PY_INCLUDE]),
Extension("gfort2py.fnumpy",["gfort2py/fnumpy.pyx"],include_dirs=[np.get_include(),PY_INCLUDE])
]				

setup(name='gfort2py',
      version='1.0.9',
      description='Python bindings for Fortran',
      license="GPLv2+",
      author='Robert Farmer',
      author_email='r.j.farmer@uva.nl',
      url='https://github.com/rjfarmer/gfort2py',
      keywords='python fortran binding',
      packages=find_packages(),
      exclude_package_data={'':['new_release.sh']},
      classifiers=[
			"Development Status :: 3 - Alpha",
			"License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
			"Programming Language :: Fortran",
		    'Programming Language :: Python :: 2.7',
		    'Programming Language :: Python :: 3',
		    'Programming Language :: Python :: 3.2',
		    'Programming Language :: Python :: 3.3',
		    'Programming Language :: Python :: 3.4',
		    'Programming Language :: Python :: 3.5',
		    'Programming Language :: Python :: 3.6',
		    'Programming Language :: Python :: 3.7',
		    'Topic :: Software Development :: Code Generators',
		    
      ],
      test_suite = 'tests',
      ext_modules=cythonize(ext),
      extras_require={
		'dev': [
			'coveralls',
			'unittest2'
			]
		}
     )



# SPDX-License-Identifier: GPL-2.0+

import ctypes

_index_t = ctypes.c_int64
_size_t = ctypes.c_int64

# Pre generate alloc array descriptors
class _bounds(ctypes.Structure):
    _fields_=[("stride",_index_t),
              ("lbound",_index_t),
              ("ubound",_index_t)]

class _fAllocArray1D(ctypes.Structure):
    ndims=1
    _fields_=[('base_addr',ctypes.c_void_p), 
              ('offset',_size_t), 
              ('dtype',_index_t),
              ('dims',_bounds*ndims)
              ]

class _fAllocArray2D(ctypes.Structure):
    ndims=2
    _fields_=[('base_addr',ctypes.c_void_p), 
              ('offset',_size_t), 
              ('dtype',_index_t),
              ('dims',_bounds*ndims)
              ]
              
class _fAllocArray3D(ctypes.Structure):
    ndims=3
    _fields_=[('base_addr',ctypes.c_void_p), 
              ('offset',_size_t), 
              ('dtype',_index_t),
              ('dims',_bounds*ndims)
              ]
              
class _fAllocArray4D(ctypes.Structure):
    ndims=4
    _fields_=[('base_addr',ctypes.c_void_p), 
              ('offset',_size_t), 
              ('dtype',_index_t),
              ('dims',_bounds*ndims)
              ]
class _fAllocArray5D(ctypes.Structure):
    ndims=5
    _fields_=[('base_addr',ctypes.c_void_p), 
              ('offset',_size_t), 
              ('dtype',_index_t),
              ('dims',_bounds*ndims)
              ]
              
class _fAllocArray6D(ctypes.Structure):
    ndims=6
    _fields_=[('base_addr',ctypes.c_void_p), 
              ('offset',_size_t), 
              ('dtype',_index_t),
              ('dims',_bounds*ndims)
              ]
              
class _fAllocArray7D(ctypes.Structure):
    ndims=7
    _fields_=[('base_addr',ctypes.c_void_p), 
              ('offset',_size_t), 
              ('dtype',_index_t),
              ('dims',_bounds*ndims)
              ]

# None is in there so we can do 1 based indexing
listFAllocArrays=[None,_fAllocArray1D,_fAllocArray2D,_fAllocArray3D,
                    _fAllocArray4D,_fAllocArray5D,_fAllocArray6D,
                    _fAllocArray7D] 



from __future__ import print_function
try:
    import __builtin__
except ImportError:
    import builtins as __builtin__

import ctypes
import numpy as np
from .errors import *

# Hacky, yes
__builtin__.quad = np.longdouble


class fVar(object):

    def __init__(self, lib, obj):
        self.__dict__.update(obj)
        self._lib = lib
        self.ctype=self.var['ctype']
        self.pytype=self.var['pytype']
        
        self._pytype = self.pytype_def()
        if self.pytype == 'quad':
            self.pytype = np.longdouble
        elif self.pytype=='bool':
            self.pytype=int
            self.ctype='c_int32'
        
        self._ctype = self.ctype_def()
        #self._ctype_f = self.ctype_def_func()

        # if true for things that are fortran things
        self._fortran = True
        
        # True if its a function argument
        self._func_arg=False
        
        #True if struct member
        self._dt_arg=False
        
        #Store the ref to the lib object
        try:   
            self._ref = self._get_from_lib()
        except NotInLib:
            self._ref = None

    def py_to_ctype(self, value):
        """
        Pass in a python value returns the ctype representation of it
        """
        return self.ctype_def()(value)
        
    def py_to_ctype_f(self, value):
        """
        Pass in a python value returns the ctype representation of it, 
        suitable for a function
        
        Second return value is anythng that needs to go at the end of the
        arg list, like a string len
        """
        x,_=self.ctype_def_func()
        return x(self.ctype_def()(value)),None

    def ctype_to_py(self, value):
        """
        Pass in a ctype value returns the python representation of it
        """
        return self.ctype_to_py_f(value)
        
    def ctype_to_py_f(self, value):
        """
        Pass in a ctype value returns the python representation of it,
        as returned by a function (may be a pointer)
        """
        if hasattr(value,'contents'):
            return self._pytype(value.contents.value)
        elif hasattr(value,'value'):
            return self._pytype(value.value)
        else:
            return self._pytype(value)


    def pytype_def(self):
        if '_cached_pytype' not in self.__dict__:
            self._cached_pytype = getattr(__builtin__, self.pytype)
        
        return self._cached_pytype

    def ctype_def(self):
        """
        The ctype type of this object
        """
        if '_cached_ctype' not in self.__dict__:
            self._cached_ctype = getattr(ctypes, self.ctype)
        
        return self._cached_ctype

    def ctype_def_func(self,pointer=False,intent=''):
        """
        The ctype type of a value suitable for use as an argument of a function

        May just call ctype_def
        
        Second return value is anything that needs to go at the end of the
        arg list, like a string len
        """
        f = ctypes.POINTER(self.ctype_def())
        if pointer:
            f = ctypes.POINTER(f)

        return f,None
        
    def py_to_ctype_p(self,value):
        """
        The ctype represnation suitable for function arguments wanting a pointer
        """

        return ctypes.POINTER(self.ctype_def())(self.py_to_ctype(value))
        

    def set_mod(self, value):
        """
        Set a module level variable
        """
        self._ref.value = self._pytype(value)

    def get(self,copy=True):
        """
        Get a module level variable
        """
        if copy:
            res = self.ctype_to_py(self._ref)
        else:
            if hasattr(r,'contents'):
                res =self._ref.contents
            else:
                res = self._ref
        
        return res

    def _get_from_lib(self):
        if 'mangled_name' in self.__dict__ and '_lib' in self.__dict__:
            try:
                return self._ctype.in_dll(self._lib, self.mangled_name)
            except ValueError:
                raise NotInLib
        raise NotInLib
        

    def _get_var_by_iter(self, value, size=-1):
        """ Gets a variable where we have to iterate to get multiple elements"""
        base_address = ctypes.addressof(value)
        return self._get_var_from_address(base_address, size=size)

    def _get_var_from_address(self, ctype_address, size=-1):
        out = []
        i = 0
        sof = ctypes.sizeof(self._ctype)
        while True:
            if i == size:
                break
            x = self._ctype.from_address(ctype_address + i * sof)
            if x.value == b'\x00':
                break
            else:
                out.append(x.value)
            i = i + 1
        return out

    def _set_var_from_iter(self, res, value, size=99999):
        base_address = ctypes.addressof(res)
        self._set_var_from_address(base_address, value, size)

    def _set_var_from_address(self, ctype_address, value, size=99999):
        for j in range(min(len(value), size)):
            offset = ctype_address + j * ctypes.sizeof(self._ctype)
            self._ctype.from_address(offset).value = value[j]

    def _pname(self):
        return str(self.name) + " <" + str(self.pytype) + ">"

    def __str__(self):
        return str(self.name)


    def __repr__(self):
        s=''
        try:
            s=str(self.get()) + " <" + str(self.pytype) + ">"
        except:
            # Skip for things that aren't in the module (function arg)
            s=" <" + str(self.pytype) + ">"
        return s
    
    def __getattr__(self, name): 
        if name in self.__dict__:
            return self.__dict__[name]
            
        if '_func_arg' in self.__dict__:
            if self._func_arg:
                return   
        
        if '_dt_arg' in self.__dict__:
            if self._dt_arg:
                return 
                
        if '_ref' in self.__dict__:
            if self._ref is None:
                return None
            else:
                try:
                    return getattr(self.get(), name)
                except:
                    return None

    #Stuff to call the result of self.get() (a python object int/str etc)

    def __add__(self, other):
        return getattr(self.get(), '__add__')(other)

    def __sub__(self, other):
        return getattr(self.get(), '__sub__')(other)

    def __mul__(self, other):
        return getattr(self.get(), '__mul__')(other)

    def __matmul__(self,other):
        return getattr(self.get(), '__matmul__')(other)

    def __truediv__(self, other):
        return getattr(self.get(), '__truediv__')(other)
        
    def __floordiv__(self,other):
        return getattr(self.get(), '__floordiv__')(other)

    def __pow__(self, other, modulo=None):
        return getattr(self.get(), '__pow__')(other,modulo)

    def __mod__(self,other):
        return getattr(self.get(), '__mod__')(other)        
        
    def __lshift__(self,other):
        return getattr(self.get(), '__lshift__')(other)        

    def __rshift__(self,other):
        return getattr(self.get(), '__rshift__')(other)

    def __and__(self,other):
        return getattr(self.get(), '__and__')(other)
        
    def __xor__(self,other):
        return getattr(self.get(), '__xor__')(other)
        
    def __or__(self,other):
        return getattr(self.get(), '__or__')(other)
        
    def __radd__(self, other):
        return getattr(self.get(), '__radd__')(other)

    def __rsub__(self, other):
        return getattr(self.get(), '__rsub__')(other)

    def __rmul__(self, other):
        return getattr(self.get(), '__rmul__')(other)

    def __rmatmul__(self,other):
        return getattr(self.get(), '__rmatmul__')(other)

    def __rtruediv__(self, other):
        return getattr(self.get(), '__rtruediv__')(other)
        
    def __rfloordiv__(self,other):
        return getattr(self.get(), '__rfloordiv__')(other)

    def __rpow__(self, other):
        return getattr(self.get(), '__rpow__')(other)

    def __rmod__(self,other):
        return getattr(self.get(), '__rmod__')(other)        
        
    def __rlshift__(self,other):
        return getattr(self.get(), '__rlshift__')(other)        

    def __rrshift__(self,other):
        return getattr(self.get(), '__rrshift__')(other)

    def __rand__(self,other):
        return getattr(self.get(), '__rand__')(other)
        
    def __rxor__(self,other):
        return getattr(self.get(), '__rxor__')(other)
        
    def __ror__(self,other):
        return getattr(self.get(), '__ror__')(other)

    def __iadd__(self, other):
        self.set_mod(self.get() + other)
        return self.get()

    def __isub__(self, other):
        self.set_mod(self.get() - other)
        return self.get()

    def __imul__(self, other):
        self.set_mod(self.get() * other)
        return self.get()

    def __itruediv__(self, other):
        self.set_mod(self.get() / other)
        return self.get()

    def __ipow__(self, other, modulo=None):
        x = self.get()**other
        if modulo:
            x = x % modulo
        self.set_mod(x)
        return self.get()

    def __eq__(self, other):
        return getattr(self.get(), '__eq__')(other)

    def __neq__(self, other):
        return getattr(self.get(), '__new__')(other)

    def __lt__(self, other):
        return getattr(self.get(), '__lt__')(other)

    def __le__(self, other):
        return getattr(self.get(), '__le__')(other)

    def __gt__(self, other):
        return getattr(self.get(), '__gt__')(other)

    def __ge__(self, other):
        return getattr(self.get(), '__ge__')(other)
        
    def __format__(self, other):
        return getattr(self.get(), '__format__')(other)
  
    def __bytes__(self):
        return getattr(self.get(), '__bytes__')()  
        
    def __bool__(self):
        return getattr(self.get(), '__bool__')()
   
    def __len__(self):
        return getattr(self.get(), '__len__')()
 
    def __length_hint__(self):
        return getattr(self.get(), '__length_hint__')()       
        
    def __dir__(self):
        return list(self.__dict__.keys()) + list(dir(self.get()))


class fParam(fVar):
    def __init__(self, lib, obj):
        self.__dict__.update(obj)
        self._lib = lib
        self.value = self.param['value']
        self.pytype = self.param['pytype']
        self._pytype = self.pytype_def()

    def set_mod(self, value):
        """
        Cant set a parameter
        """
        raise ValueError("Can't alter a parameter")

    def get(self):
        """
        A parameters value is stored in the dict, as we cant access them 
        from the shared lib.
        """
        return self._pytype(self.value)
